@@ -3,8 +3,7 @@
 # pylint: disable=protected-access
 
 
-def dump(node, ast, annotate_fields=True,
-         include_attributes=True, indent="  "):
+def dump(node, ast, annotate_fields=True, include_attributes=True, indent="  "):
   """Return a formatted dump of the tree in *node*.
 
   This is mainly useful for debugging purposes.  The returned string will show
@@ -32,19 +31,24 @@ def dump(node, ast, annotate_fields=True,
     if isinstance(node, ast.AST):
       fields = [(a, _format(b, level)) for a, b in ast.iter_fields(node)]
       if include_attributes and node._attributes:
-        fields.extend([(a, _format(getattr(node, a), level))
-                       for a in node._attributes])
+        fields.extend(
+            [(a, _format(getattr(node, a), level)) for a in node._attributes]
+        )
       return "".join([
           node.__class__.__name__,
           "(",
-          ", ".join(("%s=%s" % field for field in fields)
-                    if annotate_fields else
-                    (b for a, b in fields)),
-          ")"])
+          ", ".join(
+              ("%s=%s" % field for field in fields)
+              if annotate_fields
+              else (b for a, b in fields)
+          ),
+          ")",
+      ])
     elif isinstance(node, list):
       lines = ["["]
-      lines.extend(indent * (level + 2) + _format(x, level + 2) + ","
-                   for x in node)
+      lines.extend(
+          indent * (level + 2) + _format(x, level + 2) + "," for x in node
+      )
       if len(lines) > 1:
         lines.append(indent * (level + 1) + "]")
       else:
