@@ -23,20 +23,36 @@ import math
 import os
 import re
 import time
+import types
+
 try:
   import tracemalloc  # pylint: disable=g-import-not-at-top
 except ImportError:
   # Not available on PyPy
-  tracemalloc = None
+  tracemalloc: types.ModuleType = None
 
 
 # Metric serialization/deserialization code, taking advantage of the fact that
 # all instance vars of metrics are things that json can serialize, so we don't
 # need to write custom JsonEncoder and JsonDecoder classes per Metric subclass.
 
-# Register metric types for deserialization
+# Register metric types for deserialization.
 _METRIC_TYPES = {}
 
+# Map from metric name to Metric object.
+_registered_metrics = {}
+
+# Whether metrics should be collected.
+_enabled = False
+
+
+def reset():
+  """Resets this module to its initial state."""
+  _METRIC_TYPES.clear()
+  _registered_metrics.clear()
+  global _enabled
+  _enabled = False
+
 
 class _RegistryMeta(type):
   """Metaclass that registers subclasses in _METRIC_TYPES."""
@@ -75,9 +91,6 @@ def load_all(fp):
 
 _METRIC_NAME_RE = re.compile(r"^[a-zA-Z_]\w+$")
 
-_registered_metrics = {}  # Map from metric name to Metric object.
-_enabled = False  # True iff metrics should be collected.
-
 
 def _validate_metric_name(name):
   if _METRIC_NAME_RE.match(name) is None:
@@ -121,8 +134,9 @@ def get_metric(name, constructor, *args, **kwargs):
 
 def get_report():
   """Return a string listing all metrics, one per line."""
-  lines = [str(_registered_metrics[n]) + "\n"
-           for n in sorted(_registered_metrics)]
+  lines = [
+      str(_registered_metrics[n]) + "\n" for n in sorted(_registered_metrics)
+  ]
   return "".join(lines)
 
 
@@ -264,8 +278,9 @@ class MapCounter(Metric):
     self._total += count
 
   def _summary(self):
-    details = ", ".join(["%s=%d" % (k, self._counts[k])
-                         for k in sorted(self._counts)])
+    details = ", ".join(
+        ["%s=%d" % (k, self._counts[k]) for k in sorted(self._counts)]
+    )
     return "%d {%s}" % (self._total, details)
 
   def _merge(self, other):
@@ -306,18 +321,24 @@ class Distribution(Metric):
 
   def _stdev(self):
     if self._count:
-      variance = ((self._squared * self._count - self._total * self._total) /
-                  (self._count * self._count))
+      variance = (self._squared * self._count - self._total * self._total) / (
+          self._count * self._count
+      )
       if variance < 0.0:
         # This can only happen as the result of rounding error when the actual
         # variance is very, very close to 0.  Assume it is 0.
         return 0.0
-      return  math.sqrt(variance)
+      return math.sqrt(variance)
 
   def _summary(self):
     return "total=%s, count=%d, min=%s, max=%s, mean=%s, stdev=%s" % (
-        self._total, self._count, self._min, self._max, self._mean(),
-        self._stdev())
+        self._total,
+        self._count,
+        self._min,
+        self._max,
+        self._mean(),
+        self._stdev(),
+    )
 
   def _merge(self, other):
     # pylint: disable=protected-access
@@ -338,8 +359,9 @@ class Distribution(Metric):
 class Snapshot(Metric):
   """A metric to track memory usage via tracemalloc snapshots."""
 
-  def __init__(self, name, enabled=False, groupby="lineno",
-               nframes=1, count=10):
+  def __init__(
+      self, name, enabled=False, groupby="lineno", nframes=1, count=10
+  ):
     if enabled and tracemalloc is None:
       raise RuntimeError("tracemalloc module couldn't be imported")
     super().__init__(name)
@@ -380,8 +402,12 @@ class Snapshot(Metric):
     # doesn't take None into account during comparisons, and json will compare
     # it to None when trying to process it, causing an error. So, store it as a
     # string instead.
-    self.snapshots.append("{}:\n{}".format(where, "\n".join(
-        map(str, snap.statistics(self.groupby)[:self.count]))))
+    self.snapshots.append(
+        "{}:\n{}".format(
+            where,
+            "\n".join(map(str, snap.statistics(self.groupby)[: self.count])),
+        )
+    )
 
   def __enter__(self):
     if not self.enabled:
@@ -407,7 +433,7 @@ class MetricsContext:
 
     Args:
       output_path: The path for the metrics data.  If empty, no metrics are
-          collected.
+        collected.
       open_function: A custom file opening function.
     """
     self._output_path = output_path
