@@ -1,12 +1,13 @@
+#include <Python.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
-#include <Python.h>
 #include <structseq.h>
 
 #include <cstddef>
 #include <string>
 #include <unordered_map>
 #include <utility>
+#include <vector>
 
 #include "cfg_logging.h"
 #include "metrics.h"
@@ -46,6 +47,36 @@ static PyObject* k_next_binding_id;
 static PyObject* k_condition;
 static PyObject* k_default_data;
 
+// String constants for __dir__, Initialized on module init.
+// Program
+static PyObject *k_NewCFGNode;
+static PyObject *k_NewVariable;
+static PyObject *k_is_reachable;
+static PyObject *k_calculate_metrics;
+
+// CFGNode
+static PyObject *k_ConnectNew;
+static PyObject *k_ConnectTo;
+static PyObject *k_HasCombination;
+static PyObject *k_CanHaveCombination;
+
+// Binding
+static PyObject *k_IsVisible;
+static PyObject *k_AddOrigin;
+static PyObject *k_HasSource;
+
+// Variable
+static PyObject *k_Bindings;
+static PyObject *k_Data;
+static PyObject *k_Filter;
+static PyObject *k_FilteredData;
+static PyObject *k_AddBinding;
+static PyObject *k_PasteBindingWithNewData;
+static PyObject *k_AssignToNewVariable;
+static PyObject *k_PasteVariable;
+static PyObject *k_PasteBinding;
+
+
 typedef struct {
   PyObject_HEAD
   typegraph::Program* program;
@@ -351,9 +382,10 @@ static PyObject* NewCFGNode(PyProgramObj* self,
   }
   if (condition_obj) {
     auto bind_obj = reinterpret_cast<PyBindingObj*>(condition_obj);
-    return WrapCFGNode(self, self->program->NewCFGNode(name, bind_obj->attr));
+    return WrapCFGNode(
+        self, self->program->NewCFGNode(std::move(name), bind_obj->attr));
   }
-  return WrapCFGNode(self, self->program->NewCFGNode(name));
+  return WrapCFGNode(self, self->program->NewCFGNode(std::move(name)));
 }
 
 static std::vector<typegraph::Binding*> ParseBindingList(PyObject* list) {
@@ -416,9 +448,6 @@ static PyObject* NewVariable(PyProgramObj* self,
   if (bindings == Py_None)
     bindings = nullptr;
 
-  if (bindings && !PyObject_GetIter(bindings))
-    return nullptr;  // propagate error raised by PyObject_GetIter
-
   typegraph::CFGNode* where;
   if (!IsCFGNodeOrNone(where_obj, &where)) {
     PyErr_SetString(PyExc_TypeError, "where must be a CFGNode or None.");
@@ -434,6 +463,10 @@ static PyObject* NewVariable(PyProgramObj* self,
     CHECK(bindings && source_set && where);
     PyObject* item;
     PyObject* bind_iter = PyObject_GetIter(bindings);
+    if (bind_iter == nullptr) {
+      Py_XDECREF(source_set);
+      return nullptr;  // propagate error raised by PyObject_GetIter
+    }
     while ((item = PyIter_Next(bind_iter))) {
       // PyIter_Next returns a new reference, which will be owned by AddBinding.
       // That means we don't need to INCREF or DECREF item.
@@ -478,6 +511,30 @@ static PyObject* calculate_metrics(PyProgramObj* self, PyObject* _args) {
   return pybind11::cast(data).release().ptr();
 }
 
+
+PyDoc_STRVAR(
+    program_dir_doc,
+    "Implentation of __dir__ on Program to provide debuggability");
+
+static PyObject* ProgramDir(PyVariableObj* self, PyObject* args,
+                                      PyObject* kwargs) {
+  PyObject* list = PyList_New(0);
+  // properties
+  PyList_Append(list, k_cfg_nodes);
+  PyList_Append(list, k_variables);
+  PyList_Append(list, k_entrypoint);
+  PyList_Append(list, k_next_variable_id);
+  PyList_Append(list, k_next_binding_id);
+  PyList_Append(list, k_default_data);
+
+  // methods
+  PyList_Append(list, k_NewCFGNode);
+  PyList_Append(list, k_NewVariable);
+  PyList_Append(list, k_is_reachable);
+  PyList_Append(list, k_calculate_metrics);
+  return list;
+}
+
 static PyMethodDef program_methods[] = {
   {"NewCFGNode", reinterpret_cast<PyCFunction>(NewCFGNode),
     METH_VARARGS|METH_KEYWORDS, new_cfg_node_doc},
@@ -487,6 +544,8 @@ static PyMethodDef program_methods[] = {
    METH_VARARGS|METH_KEYWORDS, is_reachable_doc},
   {"calculate_metrics", reinterpret_cast<PyCFunction>(calculate_metrics),
    METH_NOARGS, calculate_metrics_doc},
+  {"__dir__", reinterpret_cast<PyCFunction>(ProgramDir),
+    METH_VARARGS | METH_KEYWORDS, program_dir_doc},
   {0, 0, 0, nullptr}  // sentinel
 };
 
@@ -502,51 +561,52 @@ PyDoc_STRVAR(
     "stay in memory.");
 
 PyTypeObject PyProgram = {
-  PyVarObject_HEAD_INIT(&PyType_Type, 0) tp_name : "Program",
-  tp_basicsize : sizeof(PyProgramObj),
-  tp_itemsize : 0,
-  tp_dealloc : ProgramDealloc,
+    .ob_base = PyVarObject_HEAD_INIT(&PyType_Type, 0)
+    .tp_name = "Program",
+    .tp_basicsize = sizeof(PyProgramObj),
+    .tp_itemsize = 0,
+    .tp_dealloc = ProgramDealloc,
 #if PY_VERSION_HEX >= 0x03080000  // 3.8
-  tp_vectorcall_offset: -1,
+    .tp_vectorcall_offset = -1,
 # else
-  tp_print : nullptr,
+    .tp_print = nullptr,
 # endif
-  tp_getattr : nullptr,
-  tp_setattr : nullptr,
+    .tp_getattr = nullptr,
+    .tp_setattr = nullptr,
 #if PY_VERSION_HEX >= 0x03050000  // 3.5
-  tp_as_async : nullptr,
+    .tp_as_async = nullptr,
 #else
-  tp_compare : nullptr,
+    .tp_compare = nullptr,
 #endif
-  tp_repr : nullptr,
-  tp_as_number : nullptr,
-  tp_as_sequence : nullptr,
-  tp_as_mapping : nullptr,
-  tp_hash : nullptr,
-  tp_call : nullptr,
-  tp_str : nullptr,
-  tp_getattro : ProgramGetAttro,
-  tp_setattro : ProgramSetAttro,
-  tp_as_buffer : nullptr,
-  tp_flags : 0,
-  tp_doc : program_doc,
-  tp_traverse : nullptr,
-  tp_clear : nullptr,
-  tp_richcompare : nullptr,
-  tp_weaklistoffset : 0,
-  tp_iter : nullptr,
-  tp_iternext : nullptr,
-  tp_methods : program_methods,
-  tp_members : nullptr,
-  tp_getset : nullptr,
-  tp_base : nullptr,
-  tp_dict : nullptr,
-  tp_descr_get : nullptr,
-  tp_descr_set : nullptr,
-  tp_dictoffset : 0,
-  tp_init : nullptr,
-  tp_alloc : nullptr,
-  tp_new : ProgramNew,
+    .tp_repr = nullptr,
+    .tp_as_number = nullptr,
+    .tp_as_sequence = nullptr,
+    .tp_as_mapping = nullptr,
+    .tp_hash = nullptr,
+    .tp_call = nullptr,
+    .tp_str = nullptr,
+    .tp_getattro = ProgramGetAttro,
+    .tp_setattro = ProgramSetAttro,
+    .tp_as_buffer = nullptr,
+    .tp_flags = 0,
+    .tp_doc = program_doc,
+    .tp_traverse = nullptr,
+    .tp_clear = nullptr,
+    .tp_richcompare = nullptr,
+    .tp_weaklistoffset = 0,
+    .tp_iter = nullptr,
+    .tp_iternext = nullptr,
+    .tp_methods = program_methods,
+    .tp_members = nullptr,
+    .tp_getset = nullptr,
+    .tp_base = nullptr,
+    .tp_dict = nullptr,
+    .tp_descr_get = nullptr,
+    .tp_descr_set = nullptr,
+    .tp_dictoffset = 0,
+    .tp_init = nullptr,
+    .tp_alloc = nullptr,
+    .tp_new = ProgramNew,
 };
 
 // --- CFGNode -----------------------------------------------------------------
@@ -626,25 +686,14 @@ static void CFGNodeDealloc(PyObject* self) {
 
 static PyObject* CFGNodeRepr(PyObject* self) {
   PyCFGNodeObj* py_node = reinterpret_cast<PyCFGNodeObj*>(self);
-  auto node = py_node->cfg_node;
-  PyObject* str = PyUnicode_FromFormat(
-      "<cfgnode %zu %s", node->id(), node->name().c_str());
+  typegraph::CFGNode* node = py_node->cfg_node;
   if (node->condition()) {
-    PyObject* cond_str = PyUnicode_FromFormat(
-        " condition:%zu",
-        node->condition()->variable()->id());
-    PyObject* str_cond_str = PyUnicode_Concat(str, cond_str);
-    // Drop references to the old |str| and |cond_str| as we do not need them
-    // anymore.
-    Py_DECREF(str);
-    Py_DECREF(cond_str);
-    str = str_cond_str;
+    return PyUnicode_FromFormat("<cfgnode %zu %s condition:%zu>", node->id(),
+                                node->name().c_str(),
+                                node->condition()->variable()->id());
   }
-  PyObject* final_str = PyUnicode_Concat(str, PyUnicode_FromString(">"));
-  // Drop reference to |str| as we do not need it anymore.
-  Py_DECREF(str);
-  str = final_str;
-  return str;
+  return PyUnicode_FromFormat("<cfgnode %zu %s>", node->id(),
+                              node->name().c_str());
 }
 
 PyDoc_STRVAR(connect_new_doc,
@@ -748,6 +797,30 @@ static PyObject* CanHaveCombination(PyCFGNodeObj* self,
   }
 }
 
+PyDoc_STRVAR(
+    cfg_node_dir_doc,
+    "Implentation of __dir__ on CFGNode to provide debuggability");
+
+static PyObject* CFGDir(PyVariableObj* self, PyObject* args,
+                                      PyObject* kwargs) {
+  PyObject* list = PyList_New(0);
+  // properties
+  PyList_Append(list, k_incoming);
+  PyList_Append(list, k_outgoing);
+  PyList_Append(list, k_bindings);
+  PyList_Append(list, k_name);
+  PyList_Append(list, k_program);
+  PyList_Append(list, k_id);
+  PyList_Append(list, k_condition);
+
+  // methods
+  PyList_Append(list, k_ConnectNew);
+  PyList_Append(list, k_ConnectTo);
+  PyList_Append(list, k_HasCombination);
+  PyList_Append(list, k_CanHaveCombination);
+  return list;
+}
+
 static PyMethodDef cfg_node_methods[] = {
   {"ConnectNew", reinterpret_cast<PyCFunction>(ConnectNew),
     METH_VARARGS|METH_KEYWORDS, connect_new_doc},
@@ -757,6 +830,8 @@ static PyMethodDef cfg_node_methods[] = {
     METH_VARARGS|METH_KEYWORDS, has_combination_doc},
   {"CanHaveCombination", reinterpret_cast<PyCFunction>(CanHaveCombination),
     METH_VARARGS|METH_KEYWORDS, can_have_combo_doc},
+  {"__dir__", reinterpret_cast<PyCFunction>(CFGDir),
+    METH_VARARGS | METH_KEYWORDS, cfg_node_dir_doc},
   {0, 0, 0, nullptr}  // sentinel
 };
 
@@ -767,41 +842,42 @@ PyDoc_STRVAR(cfg_node_doc,
     "inside that node.");
 
 PyTypeObject PyCFGNode = {
-  PyVarObject_HEAD_INIT(&PyType_Type, 0) tp_name : "CFGNode",
-  tp_basicsize : sizeof(PyCFGNodeObj),
-  tp_itemsize : 0,
-  tp_dealloc : CFGNodeDealloc,
+    .ob_base = PyVarObject_HEAD_INIT(&PyType_Type, 0)
+    .tp_name = "CFGNode",
+    .tp_basicsize = sizeof(PyCFGNodeObj),
+    .tp_itemsize = 0,
+    .tp_dealloc = CFGNodeDealloc,
 #if PY_VERSION_HEX >= 0x03080000  // 3.8
-  tp_vectorcall_offset: -1,
+    .tp_vectorcall_offset = -1,
 # else
-  tp_print : nullptr,
+    .tp_print = nullptr,
 # endif
-  tp_getattr : nullptr,
-  tp_setattr : nullptr,
+    .tp_getattr = nullptr,
+    .tp_setattr = nullptr,
 #if PY_VERSION_HEX >= 0x03050000  // 3.5
-  tp_as_async : nullptr,
+    .tp_as_async = nullptr,
 #else
-  tp_compare : nullptr,
+    .tp_compare = nullptr,
 #endif
-  tp_repr : CFGNodeRepr,
-  tp_as_number : nullptr,
-  tp_as_sequence : nullptr,
-  tp_as_mapping : nullptr,
-  tp_hash : nullptr,
-  tp_call : nullptr,
-  tp_str : CFGNodeRepr,
-  tp_getattro : CFGNodeGetAttro,
-  tp_setattro : CFGNodeSetAttro,
-  tp_as_buffer : nullptr,
-  tp_flags : 0,
-  tp_doc : cfg_node_doc,
-  tp_traverse : nullptr,
-  tp_clear : nullptr,
-  tp_richcompare : nullptr,
-  tp_weaklistoffset : 0,
-  tp_iter : nullptr,
-  tp_iternext : nullptr,
-  tp_methods : cfg_node_methods,
+    .tp_repr = CFGNodeRepr,
+    .tp_as_number = nullptr,
+    .tp_as_sequence = nullptr,
+    .tp_as_mapping = nullptr,
+    .tp_hash = nullptr,
+    .tp_call = nullptr,
+    .tp_str = CFGNodeRepr,
+    .tp_getattro = CFGNodeGetAttro,
+    .tp_setattro = CFGNodeSetAttro,
+    .tp_as_buffer = nullptr,
+    .tp_flags = 0,
+    .tp_doc = cfg_node_doc,
+    .tp_traverse = nullptr,
+    .tp_clear = nullptr,
+    .tp_richcompare = nullptr,
+    .tp_weaklistoffset = 0,
+    .tp_iter = nullptr,
+    .tp_iternext = nullptr,
+    .tp_methods = cfg_node_methods,
 };
 
 // --- Origin --------------------------------------------------------------
@@ -822,10 +898,10 @@ static PyStructSequence_Field origin_fields[] = {
     {nullptr}};
 
 static PyStructSequence_Desc origin_desc = {
-  name: const_cast<char*>("Origin"),
-  doc: origin_doc,
-  fields: origin_fields,
-  n_in_sequence: 2,
+    .name = const_cast<char*>("Origin"),
+    .doc = origin_doc,
+    .fields = origin_fields,
+    .n_in_sequence = 2,
 };
 
 // --- Binding ---------------------------------------------------------------
@@ -937,6 +1013,7 @@ static PyObject* AddOrigin(PyBindingObj* self, PyObject* args,
     return nullptr;
   }
   self->attr->AddOrigin(where->cfg_node, ParseBindingList(source_set));
+  Py_XDECREF(source_set);
   Py_RETURN_NONE;
 }
 
@@ -977,6 +1054,27 @@ static PyObject* HasSource(PyBindingObj* self, PyObject* args,
   }
 }
 
+PyDoc_STRVAR(
+    binding_dir_doc,
+    "Implentation of __dir__ on Binding to provide debuggability");
+
+static PyObject* BindingDir(PyVariableObj* self, PyObject* args,
+                                      PyObject* kwargs) {
+  PyObject* list = PyList_New(0);
+  // properties
+  PyList_Append(list, k_variable);
+  PyList_Append(list, k_origins);
+  PyList_Append(list, k_data);
+  PyList_Append(list, k_id);
+
+  // methods
+  PyList_Append(list, k_IsVisible);
+  PyList_Append(list, k_AddOrigin);
+  PyList_Append(list, k_AssignToNewVariable);
+  PyList_Append(list, k_HasSource);
+  return list;
+}
+
 static PyMethodDef binding_methods[] = {
     {"IsVisible", reinterpret_cast<PyCFunction>(IsVisible),
      METH_VARARGS | METH_KEYWORDS, is_visible_doc},
@@ -986,6 +1084,8 @@ static PyMethodDef binding_methods[] = {
      METH_VARARGS | METH_KEYWORDS, assign_to_new_variable_doc},
     {"HasSource", reinterpret_cast<PyCFunction>(HasSource),
      METH_VARARGS | METH_KEYWORDS, has_source_doc},
+    {"__dir__", reinterpret_cast<PyCFunction>(BindingDir),
+      METH_VARARGS | METH_KEYWORDS, binding_dir_doc},
     {0, 0, 0, nullptr},  // sentinel
 };
 
@@ -1003,41 +1103,42 @@ PyDoc_STRVAR(
     "Bindings.");
 
 PyTypeObject PyBinding = {
-  PyVarObject_HEAD_INIT(&PyType_Type, 0) tp_name : "Binding",
-  tp_basicsize : sizeof(PyBindingObj),
-  tp_itemsize : 0,
-  tp_dealloc : BindingDealloc,
+    .ob_base = PyVarObject_HEAD_INIT(&PyType_Type, 0)
+    .tp_name = "Binding",
+    .tp_basicsize = sizeof(PyBindingObj),
+    .tp_itemsize = 0,
+    .tp_dealloc = BindingDealloc,
 #if PY_VERSION_HEX >= 0x03080000  // 3.8
-  tp_vectorcall_offset: -1,
+    .tp_vectorcall_offset = -1,
 # else
-  tp_print : nullptr,
+    .tp_print = nullptr,
 # endif
-  tp_getattr : nullptr,
-  tp_setattr : nullptr,
+    .tp_getattr = nullptr,
+    .tp_setattr = nullptr,
 #if PY_VERSION_HEX >= 0x03050000  // 3.5
-  tp_as_async : nullptr,
+    .tp_as_async = nullptr,
 #else
-  tp_compare : nullptr,
+    .tp_compare = nullptr,
 #endif
-  tp_repr : BindingRepr,
-  tp_as_number : nullptr,
-  tp_as_sequence : nullptr,
-  tp_as_mapping : nullptr,
-  tp_hash : nullptr,
-  tp_call : nullptr,
-  tp_str : BindingRepr,
-  tp_getattro : BindingGetAttro,
-  tp_setattro : nullptr,
-  tp_as_buffer : nullptr,
-  tp_flags : 0,
-  tp_doc : binding_doc,
-  tp_traverse : nullptr,
-  tp_clear : nullptr,
-  tp_richcompare : nullptr,
-  tp_weaklistoffset : 0,
-  tp_iter : nullptr,
-  tp_iternext : nullptr,
-  tp_methods : binding_methods,
+    .tp_repr = BindingRepr,
+    .tp_as_number = nullptr,
+    .tp_as_sequence = nullptr,
+    .tp_as_mapping = nullptr,
+    .tp_hash = nullptr,
+    .tp_call = nullptr,
+    .tp_str = BindingRepr,
+    .tp_getattro = BindingGetAttro,
+    .tp_setattro = nullptr,
+    .tp_as_buffer = nullptr,
+    .tp_flags = 0,
+    .tp_doc = binding_doc,
+    .tp_traverse = nullptr,
+    .tp_clear = nullptr,
+    .tp_richcompare = nullptr,
+    .tp_weaklistoffset = 0,
+    .tp_iter = nullptr,
+    .tp_iternext = nullptr,
+    .tp_methods = binding_methods,
 };
 
 // --- Variable ----------------------------------------------------------------
@@ -1093,7 +1194,7 @@ static int VariableSetAttro(PyObject* self, PyObject* attr, PyObject* val) {
 
 PyDoc_STRVAR(
     variable_prune_doc,
-    "Bindings(cfg_node, strict=True)\n\n"
+    "Bindings(cfg_node)\n\n"
     "Filters down the possibilities of bindings for this variable, by "
     "analyzing "
     "the control flow graph. Any definition for this variable that is "
@@ -1103,19 +1204,16 @@ PyDoc_STRVAR(
 
 static PyObject* VariablePrune(PyVariableObj* self,
                                PyObject* args, PyObject* kwargs) {
-  static const char *kwlist[] = {"cfg_node", "strict", nullptr};
+  static const char *kwlist[] = {"cfg_node", nullptr};
   PyObject* cfg_node_obj;
-  PyObject* strict_obj = nullptr;
-  if (!SafeParseTupleAndKeywords(args, kwargs, "O|O", kwlist, &cfg_node_obj,
-                                 &strict_obj))
+  if (!SafeParseTupleAndKeywords(args, kwargs, "O", kwlist, &cfg_node_obj))
     return nullptr;
   typegraph::CFGNode* cfg_node = nullptr;
   if (!IsCFGNodeOrNone(cfg_node_obj, &cfg_node)) {
     PyErr_SetString(PyExc_TypeError, "where must be a CFGNode or None.");
     return nullptr;
   }
-  const auto strict = IsTruthy(strict_obj);
-  auto bindings = self->u->Prune(cfg_node, strict);
+  auto bindings = self->u->Prune(cfg_node);
   PyObject* list = PyList_New(0);
   PyProgramObj* program = get_program(self);
   for (typegraph::Binding* attr : bindings) {
@@ -1203,7 +1301,7 @@ static PyObject* VariableFilteredData(PyVariableObj* self,
   return list;
 }
 
-PyDoc_STRVAR(variable_add_choice_doc,
+PyDoc_STRVAR(variable_add_binding_doc,
              "AddBinding(data, source_set, where)\n\n"
              "Adds another option to this variable.\n\n"
              "This will not overwrite this variable in the current CFGNode. "
@@ -1250,24 +1348,29 @@ static PyObject* VariableAddBinding(PyVariableObj* self, PyObject* args,
   return WrapBinding(program, attr);
 }
 
-PyDoc_STRVAR(variable_add_bindings_doc,
-             "AddBindings(variable)\n\n"
-             "Adds all the Bindings from another variable to this one.");
+PyDoc_STRVAR(
+    variable_paste_binding_with_new_data_doc,
+    "Add data to this variable with origins copied from the given binding.\n\n"
+    "When copying a binding from one variable to another *at the same node*, "
+    "use this method to preseve the origins while changing the data. "
+    "Conceptually, you would want to do this when 'binding.data' and 'data' "
+    "represent the same data (but 'data' might, say, be a simplified "
+    "representation).");
 
-static PyObject* VariableAddBindings(PyVariableObj* self, PyObject* args,
-                                     PyObject* kwargs) {
-  static const char *kwlist[] = {"variable", "where", nullptr};
-  PyVariableObj* variable = nullptr;
-  PyCFGNodeObj* where = nullptr;
-  if (!SafeParseTupleAndKeywords(args, kwargs, "O!O!", kwlist, &PyVariable,
-                                 &variable, &PyCFGNode, &where)) {
+static PyObject* VariablePasteBindingWithNewData(
+    PyVariableObj* self, PyObject* args, PyObject* kwargs) {
+  static const char* kwlist[] = {"binding", "data", nullptr};
+  PyBindingObj* binding;
+  PyObject* data = nullptr;
+  if (!SafeParseTupleAndKeywords(args, kwargs, "O!O", kwlist, &PyBinding,
+                                 &binding, &data)) {
     return nullptr;
   }
-  for (const auto& binding : variable->u->bindings()) {
-    typegraph::Binding* copy = self->u->AddBinding(binding->data());
-    copy->CopyOrigins(binding.get(), where->cfg_node);
-  }
-  Py_RETURN_NONE;
+  PyProgramObj* program = get_program(self);
+  Py_INCREF(data);
+  typegraph::Binding* attr = self->u->PasteBindingWithNewData(
+      binding->attr, MakeBindingData(data));
+  return WrapBinding(program, attr);
 }
 
 PyDoc_STRVAR(
@@ -1370,6 +1473,32 @@ static PyObject* VariablePasteBinding(PyVariableObj* self, PyObject* args,
   Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(
+    variable_dir_doc,
+    "Implentation of __dir__ on Variable to provide debuggability");
+
+static PyObject* VariableDir(PyVariableObj* self, PyObject* args,
+                                      PyObject* kwargs) {
+  PyObject* list = PyList_New(0);
+  // properties
+  PyList_Append(list, k_bindings);
+  PyList_Append(list, k_data);
+  PyList_Append(list, k_id);
+  PyList_Append(list, k_program);
+
+  // methods
+  PyList_Append(list, k_Bindings);
+  PyList_Append(list, k_Data);
+  PyList_Append(list, k_Filter);
+  PyList_Append(list, k_FilteredData);
+  PyList_Append(list, k_AddBinding);
+  PyList_Append(list, k_PasteBindingWithNewData);
+  PyList_Append(list, k_AssignToNewVariable);
+  PyList_Append(list, k_PasteVariable);
+  PyList_Append(list, k_PasteBinding);
+  return list;
+}
+
 static PyMethodDef variable_methods[] = {
     {"Bindings", reinterpret_cast<PyCFunction>(VariablePrune),
      METH_VARARGS | METH_KEYWORDS, variable_prune_doc},
@@ -1380,9 +1509,10 @@ static PyMethodDef variable_methods[] = {
     {"FilteredData", reinterpret_cast<PyCFunction>(VariableFilteredData),
      METH_VARARGS | METH_KEYWORDS, variable_filtered_data_doc},
     {"AddBinding", reinterpret_cast<PyCFunction>(VariableAddBinding),
-     METH_VARARGS | METH_KEYWORDS, variable_add_choice_doc},
-    {"AddBindings", reinterpret_cast<PyCFunction>(VariableAddBindings),
-     METH_VARARGS | METH_KEYWORDS, variable_add_bindings_doc},
+     METH_VARARGS | METH_KEYWORDS, variable_add_binding_doc},
+    {"PasteBindingWithNewData",
+     reinterpret_cast<PyCFunction>(VariablePasteBindingWithNewData),
+     METH_VARARGS | METH_KEYWORDS, variable_paste_binding_with_new_data_doc},
     {"AssignToNewVariable",
      reinterpret_cast<PyCFunction>(VarAssignToNewVariable),
      METH_VARARGS | METH_KEYWORDS, var_assign_to_new_variable_doc},
@@ -1390,6 +1520,8 @@ static PyMethodDef variable_methods[] = {
       METH_VARARGS | METH_KEYWORDS, variable_paste_variable_doc},
     {"PasteBinding", reinterpret_cast<PyCFunction>(VariablePasteBinding),
       METH_VARARGS | METH_KEYWORDS, variable_paste_binding_doc},
+    {"__dir__", reinterpret_cast<PyCFunction>(VariableDir),
+      METH_VARARGS | METH_KEYWORDS, variable_dir_doc},
     {0, 0, 0, nullptr},  // sentinel
 };
 
@@ -1402,41 +1534,42 @@ PyDoc_STRVAR(
     "PasteVariable.");
 
 PyTypeObject PyVariable = {
-  PyVarObject_HEAD_INIT(&PyType_Type, 0) tp_name : "Variable",
-  tp_basicsize : sizeof(PyVariableObj),
-  tp_itemsize : 0,
-  tp_dealloc : VariableDealloc,
+    .ob_base = PyVarObject_HEAD_INIT(&PyType_Type, 0)
+    .tp_name = "Variable",
+    .tp_basicsize = sizeof(PyVariableObj),
+    .tp_itemsize = 0,
+    .tp_dealloc = VariableDealloc,
 #if PY_VERSION_HEX >= 0x03080000  // 3.8
-  tp_vectorcall_offset: -1,
+    .tp_vectorcall_offset = -1,
 # else
-  tp_print : nullptr,
+    .tp_print = nullptr,
 # endif
-  tp_getattr : nullptr,
-  tp_setattr : nullptr,
+    .tp_getattr = nullptr,
+    .tp_setattr = nullptr,
 #if PY_VERSION_HEX >= 0x03050000  // 3.5
-  tp_as_async : nullptr,
+    .tp_as_async = nullptr,
 #else
-  tp_compare : nullptr,
+    .tp_compare = nullptr,
 #endif
-  tp_repr : VariableRepr,
-  tp_as_number : nullptr,
-  tp_as_sequence : nullptr,
-  tp_as_mapping : nullptr,
-  tp_hash : nullptr,
-  tp_call : nullptr,
-  tp_str : VariableRepr,
-  tp_getattro : VariableGetAttro,
-  tp_setattro : VariableSetAttro,
-  tp_as_buffer : nullptr,
-  tp_flags : 0,
-  tp_doc : variable_doc,
-  tp_traverse : nullptr,
-  tp_clear : nullptr,
-  tp_richcompare : nullptr,
-  tp_weaklistoffset : 0,
-  tp_iter : nullptr,
-  tp_iternext : nullptr,
-  tp_methods : variable_methods,
+    .tp_repr = VariableRepr,
+    .tp_as_number = nullptr,
+    .tp_as_sequence = nullptr,
+    .tp_as_mapping = nullptr,
+    .tp_hash = nullptr,
+    .tp_call = nullptr,
+    .tp_str = VariableRepr,
+    .tp_getattro = VariableGetAttro,
+    .tp_setattro = VariableSetAttro,
+    .tp_as_buffer = nullptr,
+    .tp_flags = 0,
+    .tp_doc = variable_doc,
+    .tp_traverse = nullptr,
+    .tp_clear = nullptr,
+    .tp_richcompare = nullptr,
+    .tp_weaklistoffset = 0,
+    .tp_iter = nullptr,
+    .tp_iternext = nullptr,
+    .tp_methods = variable_methods,
 };
 
 // --- cfg module and metrics --------------------------------------------------
@@ -1498,6 +1631,51 @@ static PyObject* InitModule(PyObject* module) {
   k_condition = PyUnicode_FromString("condition");
   Py_XDECREF(k_default_data);
   k_default_data = PyUnicode_FromString("default_data");
+
+  // Program
+  Py_XDECREF(k_NewCFGNode);
+  k_NewCFGNode = PyUnicode_FromString("NewCFGNode");
+  Py_XDECREF(k_NewVariable);
+  k_NewVariable = PyUnicode_FromString("NewVariable");
+  Py_XDECREF(k_is_reachable);
+  k_is_reachable = PyUnicode_FromString("is_reachable");
+  Py_XDECREF(k_calculate_metrics);
+  k_calculate_metrics = PyUnicode_FromString("calculate_metrics");
+  // CFGNode
+  Py_XDECREF(k_ConnectNew);
+  k_ConnectNew = PyUnicode_FromString("ConnectNew");
+  Py_XDECREF(k_ConnectTo);
+  k_ConnectTo = PyUnicode_FromString("ConnectTo");
+  Py_XDECREF(k_HasCombination);
+  k_HasCombination = PyUnicode_FromString("HasCombination");
+  Py_XDECREF(k_CanHaveCombination);
+  k_CanHaveCombination = PyUnicode_FromString("CanHaveCombination");
+  // Binding
+  Py_XDECREF(k_IsVisible);
+  k_IsVisible = PyUnicode_FromString("IsVisible");
+  Py_XDECREF(k_AddOrigin);
+  k_AddOrigin = PyUnicode_FromString("AddOrigin");
+  Py_XDECREF(k_HasSource);
+  k_HasSource = PyUnicode_FromString("HasSource");
+  // Variable
+  Py_XDECREF(k_Bindings);
+  k_Bindings = PyUnicode_FromString("Bindings");
+  Py_XDECREF(k_Data);
+  k_Data = PyUnicode_FromString("Data");
+  Py_XDECREF(k_Filter);
+  k_Filter = PyUnicode_FromString("Filter");
+  Py_XDECREF(k_FilteredData);
+  k_FilteredData = PyUnicode_FromString("FilteredData");
+  Py_XDECREF(k_AddBinding);
+  k_AddBinding = PyUnicode_FromString("AddBinding");
+  Py_XDECREF(k_PasteBindingWithNewData);
+  k_PasteBindingWithNewData = PyUnicode_FromString("PasteBindingWithNewData");
+  Py_XDECREF(k_AssignToNewVariable);
+  k_AssignToNewVariable = PyUnicode_FromString("AssignToNewVariable");
+  Py_XDECREF(k_PasteVariable);
+  k_PasteVariable = PyUnicode_FromString("PasteVariable");
+  Py_XDECREF(k_PasteBinding);
+  k_PasteBinding = PyUnicode_FromString("PasteBinding");
   return module;
 }
 
