@@ -2,11 +2,16 @@
 
 #include <algorithm>
 #include <cstddef>
+#include <deque>
 #include <functional>
 #include <iterator>
-#include <stack>
+#include <memory>
+#include <optional>
+#include <set>
 #include <string>
 #include <tuple>
+#include <unordered_map>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -19,20 +24,6 @@ namespace devtools_python_typegraph {
 
 namespace internal {
 
-// Helper function for checking set membership.
-template <class T, class U>
-static bool set_contains(const std::set<T, U>& set, const T elem) {
-  return set.find(elem) != set.end();
-}
-
-static bool node_set_contains(const CFGNodeSet& set, const CFGNode* elem) {
-  return set_contains<const CFGNode*>(set, elem);
-}
-
-static bool goal_set_contains(const GoalSet& set, const Binding* elem) {
-  return set_contains<const Binding*>(set, elem);
-}
-
 // Stores the results of remove_finished_goals.
 struct RemoveResult {
   const GoalSet removed_goals;
@@ -52,7 +43,7 @@ static std::vector<RemoveResult> remove_finished_goals(const CFGNode* pos,
   GoalSet goals_to_remove;
   // We can't use set_intersection here because pos->bindings() is a vector.
   for (const auto* goal : pos->bindings()) {
-    if (goal_set_contains(goals, goal)) {
+    if (goals.count(goal)) {
       goals_to_remove.insert(goal);
     }
   }
@@ -64,8 +55,7 @@ static std::vector<RemoveResult> remove_finished_goals(const CFGNode* pos,
                       std::inserter(new_goals, new_goals.begin()),
                       pointer_less<Binding>());
   std::vector<std::tuple<GoalSet, GoalSet, GoalSet, GoalSet>> stack;
-  stack.push_back(
-      std::make_tuple(goals_to_remove, seen_goals, removed_goals, new_goals));
+  stack.emplace_back(goals_to_remove, seen_goals, removed_goals, new_goals);
   std::vector<RemoveResult> results;
   while (!stack.empty()) {
     std::tie(
@@ -77,33 +67,31 @@ static std::vector<RemoveResult> remove_finished_goals(const CFGNode* pos,
     }
     const auto* goal = *goals_to_remove.begin();
     goals_to_remove.erase(goals_to_remove.begin());
-    if (goal_set_contains(seen_goals, goal)) {
+    if (seen_goals.count(goal)) {
       // Only process a goal once, to prevent infinite loops.
-      stack.push_back(std::make_tuple(
-          goals_to_remove, seen_goals, removed_goals, new_goals));
+      stack.emplace_back(goals_to_remove, seen_goals, removed_goals, new_goals);
       continue;
     }
     seen_goals.insert(goal);
     const auto* origin = goal->FindOrigin(pos);
     if (!origin) {
       new_goals.insert(goal);
-      stack.push_back(std::make_tuple(
-          goals_to_remove, seen_goals, removed_goals, new_goals));
+      stack.emplace_back(goals_to_remove, seen_goals, removed_goals, new_goals);
       continue;
     }
     removed_goals.insert(goal);
     for (const auto& source_set : origin->source_sets) {
       GoalSet next_goals_to_remove(goals_to_remove);
       next_goals_to_remove.insert(source_set.begin(), source_set.end());
-      stack.push_back(std::make_tuple(
-          next_goals_to_remove, seen_goals, removed_goals, new_goals));
+      stack.emplace_back(std::move(next_goals_to_remove), seen_goals,
+                         removed_goals, new_goals);
     }
   }
   return results;
 }
 
-State::State(const CFGNode* pos, const GoalSet& goals)
-    : pos_(pos), goals_(goals) {}
+State::State(const CFGNode* pos, GoalSet goals)
+    : pos_(pos), goals_(std::move(goals)) {}
 
 State::State(const CFGNode* pos, const std::vector<const Binding*>& goals)
     : pos_(pos), goals_(goals.begin(), goals.end()) {}
@@ -117,11 +105,51 @@ std::size_t State::Hash() const {
   return hash;
 }
 
-PathFinder::PathFinder()
-    : solved_find_queries_(
-          new QueryMap){}
+QueryResult PathCacheTrie::InsertResult(
+    const CFGNode* start, const CFGNode* finish, const CFGNodeSet& blocked,
+    bool path_exists, std::deque<const CFGNode*> result_path) {
+  TrieNode* current_trie_node = &root_[start][finish];
+  std::unordered_map<CFGNode::IdType, std::unique_ptr<TrieNode>>*
+      current_children = &current_trie_node->children;
 
-PathFinder::~PathFinder() {}
+  for (const CFGNode* node : blocked) {
+    auto it = current_children->find(node->id());
+    if (it == current_children->end()) {
+      auto inserted =
+          current_children->insert({node->id(), std::make_unique<TrieNode>()});
+      current_trie_node = inserted.first->second.get();
+      current_children = &current_trie_node->children;
+      continue;
+    }
+    current_trie_node = it->second.get();
+    current_children = &it->second->children;
+  }
+  current_trie_node->path = std::move(result_path);
+  current_trie_node->path_exists = path_exists;
+  return {path_exists, &current_trie_node->path.value()};
+}
+
+QueryResult PathCacheTrie::GetResult(const CFGNode* start,
+                                     const CFGNode* finish,
+                                     const CFGNodeSet& blocked) {
+  TrieNode* current_trie_node = &root_[start][finish];
+  std::unordered_map<CFGNode::IdType, std::unique_ptr<TrieNode>>*
+      current_children = &current_trie_node->children;
+
+  for (const CFGNode* node : blocked) {
+    auto it = current_children->find(node->id());
+    if (it == current_children->end()) {
+      return {false, nullptr};
+    }
+    current_trie_node = it->second.get();
+    current_children = &it->second->children;
+  }
+
+  if (current_trie_node->path.has_value()) {
+    return {current_trie_node->path_exists, &current_trie_node->path.value()};
+  }
+  return {false, nullptr};
+}
 
 bool PathFinder::FindAnyPathToNode(
     const CFGNode* start,
@@ -136,7 +164,7 @@ bool PathFinder::FindAnyPathToNode(
     stack.pop_back();
     if (node == finish)
       return true;
-    if (node_set_contains(seen, node) || node_set_contains(blocked, node))
+    if (seen.count(node) || blocked.count(node))
       continue;
     seen.insert(node);
     stack.insert(stack.end(), node->incoming().begin(), node->incoming().end());
@@ -144,7 +172,7 @@ bool PathFinder::FindAnyPathToNode(
   return false;
 }
 
-const std::deque<const CFGNode*> PathFinder::FindShortestPathToNode(
+std::deque<const CFGNode*> PathFinder::FindShortestPathToNode(
     const CFGNode* start, const CFGNode* finish,
     const CFGNodeSet& blocked) const {
   std::deque<const CFGNode*> queue;
@@ -161,11 +189,12 @@ const std::deque<const CFGNode*> PathFinder::FindShortestPathToNode(
       found = true;
       break;
     }
-    if (node_set_contains(seen, node) || node_set_contains(blocked, node))
+    if (seen.count(node) || blocked.count(node))
       continue;
     seen.insert(node);
-    for (auto n : node->incoming())
-      previous.insert({n, node});
+    for (auto n : node->incoming()) {
+      previous.emplace(n, node);
+    }
     queue.insert(queue.end(), node->incoming().begin(), node->incoming().end());
   }
   std::deque<const CFGNode*> path;
@@ -180,13 +209,13 @@ const std::deque<const CFGNode*> PathFinder::FindShortestPathToNode(
 }
 
 const CFGNode* PathFinder::FindHighestReachableWeight(
-    const CFGNode* start, CFGNodeSet seen,
+    const CFGNode* start,
+    std::unordered_set<const CFGNode*, CFGNodePtrHash>& seen,
     const std::unordered_map<const CFGNode*, int, CFGNodePtrHash>& weight_map)
     const {
   std::vector<const CFGNode*> stack;
   stack.insert(stack.end(), start->incoming().begin(), start->incoming().end());
   int best_weight = -1;
-  int weight;
   const CFGNode* best_node = nullptr;
   const CFGNode* node;
   while (!stack.empty()) {
@@ -195,35 +224,29 @@ const CFGNode* PathFinder::FindHighestReachableWeight(
     if (node == start)
       // Don't allow loops back to the start.
       continue;
-    weight = map_util::FindOrDefault(weight_map, node, -1);
-    if (weight > best_weight) {
-      best_weight = weight;
+    const auto weight = weight_map.find(node);
+    if (weight != weight_map.end() && weight->second > best_weight) {
+      best_weight = weight->second;
       best_node = node;
     }
-    if (node_set_contains(seen, node))
-      continue;
-    seen.insert(node);
-  stack.insert(stack.end(), node->incoming().begin(), node->incoming().end());
+    if (!seen.insert(node).second) continue;
+    stack.insert(stack.end(), node->incoming().begin(), node->incoming().end());
   }
   return best_node;
 }
 
-QueryResult PathFinder::FindNodeBackwards(
-    const CFGNode* start,
-    const CFGNode* finish,
-    const CFGNodeSet& blocked) {
-  QueryKey query(start, finish, blocked);
-  const auto* res = map_util::FindOrNull(*solved_find_queries_, query);
-  if (res)
-    return *res;
-  // Declaring result here and filling it in later lets us use RVO.
-  QueryResult result;
+QueryResult PathFinder::FindNodeBackwards(const CFGNode* start,
+                                          const CFGNode* finish,
+                                          const CFGNodeSet& blocked) {
+  QueryResult result = path_trie_.GetResult(start, finish, blocked);
+  if (result.path) {
+    return result;
+  }
   auto shortest_path = FindShortestPathToNode(start, finish, blocked);
   if (shortest_path.empty()) {
-    result.path_exists = false;
-    result.path = shortest_path;
-    (*solved_find_queries_)[query] = result;
-    return result;
+    return path_trie_.InsertResult(start, finish, blocked,
+                                   /*path_exists=*/false,
+                                   std::move(shortest_path));
   }
   // We now have the shortest path to finish. All articulation points are
   // guaranteed to be on that path (since they're on *all* possible paths).
@@ -231,13 +254,14 @@ QueryResult PathFinder::FindNodeBackwards(
   // without using any nodes on it. The furthest node we can reach (described
   // below by the "weight", which is the position on our shortest path) is our
   // first articulation point. Set that as new start and continue.
-  CFGNodeSet blocked_(blocked);
+  std::unordered_set<const CFGNode*, CFGNodePtrHash> blocked_;
+  blocked_.insert(blocked.begin(), blocked.end());
   blocked_.insert(shortest_path.begin(), shortest_path.end());
   std::unordered_map<const CFGNode*, int, CFGNodePtrHash> weights;
   int w = 0;
-  std::deque<const CFGNode*>::const_iterator it = shortest_path.cbegin();
-  for (; it != shortest_path.cend(); w++, it++)
-    weights[*it] = w;
+  for (const auto& x : shortest_path) {
+    weights[x] = w++;
+  }
   std::deque<const CFGNode*> path;
   const CFGNode* node = start;
   while (true) {
@@ -247,38 +271,30 @@ QueryResult PathFinder::FindNodeBackwards(
       break;
     node = FindHighestReachableWeight(node, blocked_, weights);
   }
-  result.path_exists = true;
-  result.path = path;
-  (*solved_find_queries_)[query] = result;
-  return result;
+  return path_trie_.InsertResult(start, finish, blocked, true, std::move(path));
 }
 
 }  // namespace internal
 
 Solver::Solver(const Program* program)
-    : solved_states_(new internal::StateMap),
-      state_cache_hits_(0),
-      state_cache_misses_(0),
-      program_(program) {}
+    : state_cache_hits_(0), state_cache_misses_(0), program_(program) {}
 
 SolverMetrics Solver::CalculateMetrics() const {
-  auto cm = CacheMetrics(solved_states_->size(), state_cache_hits_,
+  auto cm = CacheMetrics(solved_states_.size(), state_cache_hits_,
                          state_cache_misses_);
-  return SolverMetrics(std::vector<QueryMetrics>(query_metrics_), cm);
+  return SolverMetrics(query_metrics_, std::move(cm));
 }
 
 bool Solver::GoalsConflict(const internal::GoalSet& goals) const {
   std::unordered_map<const Variable*, const Binding*> variables;
   for (const Binding* goal : goals) {
-    const Binding* existing = map_util::FindPtrOrNull(variables,
-                                                      goal->variable());
-    if (existing) {
-      CHECK(existing != goal) << "Internal error. Duplicate goal.";
-      CHECK(existing->data() != goal->data()) <<
-          "Internal error. Duplicate data across bindings.";
+    const auto& [it, inserted] = variables.emplace(goal->variable(), goal);
+    if (!inserted) {
+      CHECK(it->second != goal) << "Internal error. Duplicate goal.";
+      CHECK(it->second->data() != goal->data())
+          << "Internal error. Duplicate data across bindings.";
       return true;
     }
-    variables[goal->variable()] = goal;
   }
   return false;
 }
@@ -302,7 +318,7 @@ bool Solver::FindSolution(const internal::State& state,
   for (const Binding* goal : state.goals()) goal_ids.push_back(goal->id());
 
   query_metrics_.back().add_step(
-      QueryStep(state.pos()->id(), goal_ids, current_depth));
+      QueryStep(state.pos()->id(), std::move(goal_ids), current_depth));
   for (const Binding* goal : state.goals()) {
     LOG(INFO) << indent << "Goal: " << goal->variable()->id() << " = "
               << goal->data();
@@ -342,29 +358,36 @@ bool Solver::FindSolution(const internal::State& state,
       blocked.insert(vnodes.begin(), vnodes.end());
     }
     CFGNodeSet new_positions;
+    // calling FindNodeBackwards is expensive, even with the trie cache going on
+    // there, because trie queries can happen tons of time if the CFG gets
+    // complicated. thus gather unique sets of nodes that want to visit first to
+    // avoid querying for the same path multiple times.
+    std::unordered_set<const CFGNode*> unique_finish_nodes;
     for (const Binding* goal : result.new_goals) {
       for (const auto& origin : goal->origins()) {
-        internal::QueryResult origin_path = path_finder_.FindNodeBackwards(
-            state.pos(), origin->where, blocked);
-        if (origin_path.path_exists) {
-          const CFGNode* where = origin->where;
-          // Check if we found conditions on the way.
-          for (const CFGNode* node : origin_path.path) {
-            if (node != state.pos()) {
-              where = node;
-              break;
-            }
+        unique_finish_nodes.insert(origin->where);
+      }
+    }
+    for (const CFGNode* finish_node : unique_finish_nodes) {
+      internal::QueryResult origin_path =
+          path_finder_.FindNodeBackwards(state.pos(), finish_node, blocked);
+      if (origin_path.path_exists) {
+        const CFGNode* where = finish_node;
+        // Check if we found conditions on the way.
+        for (const CFGNode* node : *origin_path.path) {
+          if (node != state.pos()) {
+            where = node;
+            break;
           }
-          new_positions.insert(where);
         }
+        new_positions.insert(where);
       }
     }
     for (const auto* new_pos : new_positions) {
       LOG(INFO) << indent << "New pos: <" << new_pos->id() << "> "
                 << new_pos->name();
       const internal::State new_state(new_pos, result.new_goals);
-      if (internal::set_contains<const internal::State*>(
-              seen_states, &new_state) && new_positions.size() > 1) {
+      if (seen_states.count(&new_state) > 0 && new_positions.size() > 1) {
         // Cycle detected. We ignore it unless it is the only solution.
         continue;
       }
@@ -398,7 +421,7 @@ bool Solver::CanHaveSolution(
 bool Solver::RecallOrFindSolution(
     const internal::State& state, internal::StateSet& seen_states,
     int current_depth) {
-  const bool* status = map_util::FindOrNull(*solved_states_, state);
+  const bool* status = map_util::FindOrNull(solved_states_, state);
   if (status) {
     state_cache_hits_ += 1;
     query_metrics_.back().set_from_cache(true);
@@ -417,14 +440,15 @@ bool Solver::RecallOrFindSolution(
   // solvable state, even though we have not solved it yet. The reasoning is
   // that if it's possible to solve this state at this level of the tree, it can
   // also be solved in any of the children.
-  (*solved_states_)[state] = true;
+  solved_states_[state] = true;
   // Careful! Modifying seen_states would affect other recursive calls, so we
   // need to copy it.
-  internal::StateSet new_seen_states(seen_states);
-  new_seen_states.insert(&state);
-
-  bool result = FindSolution(state, new_seen_states, current_depth);
-  (*solved_states_)[state] = result;
+  auto inserted = seen_states.insert(&state);
+  bool result = FindSolution(state, seen_states, current_depth);
+  solved_states_[state] = result;
+  if (inserted.second) {
+    seen_states.erase(inserted.first);
+  }
   return result;
 }
 
