@@ -53,7 +53,7 @@ class VmTestBase(test_base.BaseTest, test_utils.MakeCodeMixin):
     self.ctx = self.make_context()
 
   def make_context(self):
-    return context.Context(options=self.options, loader=self.loader)
+    return context.Context(options=self.options, loader=self.loader, src="")
 
 
 class TraceVmTestBase(VmTestBase):
@@ -69,7 +69,7 @@ class TraceTest(TraceVmTestBase):
 
   def test_empty_data(self):
     """Test that we can trace values without data."""
-    op = test_utils.FakeOpcode("foo.py", 123, "foo")
+    op = test_utils.FakeOpcode("foo.py", 123, 123, 0, 0, "foo")
     self.ctx.vm.trace_opcode(op, "x", 42)
     self.assertEqual(self.ctx.vm.opcode_traces, [(op, "x", (None,))])
 
@@ -78,25 +78,44 @@ class TraceTest(TraceVmTestBase):
       x = 1  # line 1
       y = x  # line 2
     """).lstrip()
-    # Compiles to:
-    #     0 LOAD_CONST     0 (1)
-    #     3 STORE_NAME     0 (x)
-    #
-    #     6 LOAD_NAME      0 (x)
-    #     9 STORE_NAME     1 (y)
-    #    12 LOAD_CONST     1 (None)
-    #    15 RETURN_VALUE
+    if self.ctx.python_version >= (3, 12):
+      # Compiles to:
+      #     2 LOAD_CONST     0 (1)
+      #     4 STORE_NAME     0 (x)
+      #
+      #     6 LOAD_NAME      0 (x)
+      #     8 STORE_NAME     1 (y)
+      #    10 RETURN_CONST   1 (None)
+      expected = [
+          # (opcode, line number, symbol)
+          ("LOAD_CONST", 1, 1),
+          ("STORE_NAME", 1, "x"),
+          ("LOAD_NAME", 2, "x"),
+          ("STORE_NAME", 2, "y"),
+          ("RETURN_CONST", 2, None),
+      ]
+    else:
+      # Compiles to:
+      #     0 LOAD_CONST     0 (1)
+      #     3 STORE_NAME     0 (x)
+      #
+      #     6 LOAD_NAME      0 (x)
+      #     9 STORE_NAME     1 (y)
+      #    12 LOAD_CONST     1 (None)
+      #    15 RETURN_VALUE
+      expected = [
+          # (opcode, line number, symbol)
+          ("LOAD_CONST", 1, 1),
+          ("STORE_NAME", 1, "x"),
+          ("LOAD_NAME", 2, "x"),
+          ("STORE_NAME", 2, "y"),
+          ("LOAD_CONST", 2, None),
+      ]
     self.ctx.vm.run_program(src, "", maximum_depth=10)
-    expected = [
-        # (opcode, line number, symbol)
-        ("LOAD_CONST", 1, 1),
-        ("STORE_NAME", 1, "x"),
-        ("LOAD_NAME", 2, "x"),
-        ("STORE_NAME", 2, "y"),
-        ("LOAD_CONST", 2, None)
+    actual = [
+        (op.name, op.line, symbol)
+        for op, symbol, _ in self.ctx.vm.opcode_traces
     ]
-    actual = [(op.name, op.line, symbol)
-              for op, symbol, _ in self.ctx.vm.opcode_traces]
     self.assertEqual(actual, expected)
 
 
@@ -106,12 +125,14 @@ class AnnotationsTest(VmTestBase):
   def test_record_local_ops(self):
     self.ctx.vm.run_program("v: int = None", "", maximum_depth=10)
     self.assertEqual(
-        self.ctx.vm.local_ops, {
+        self.ctx.vm.local_ops,
+        {
             "<module>": [
                 vm.LocalOp(name="v", op=vm.LocalOp.Op.ASSIGN),
-                vm.LocalOp(name="v", op=vm.LocalOp.Op.ANNOTATE)
+                vm.LocalOp(name="v", op=vm.LocalOp.Op.ANNOTATE),
             ]
-        })
+        },
+    )
 
 
 if __name__ == "__main__":
